#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include "BSState.h"

BSState::BSState(int nchairs)
{
    int ii;

    barberIsSleeping = 0;
    customerInBarberChair = NO_CUSTOMER;
    nWaitingChairs = nchairs;
    customerInWaitingChair = (int *)malloc(nchairs * sizeof(int));
    for(ii = 0; ii < nWaitingChairs; ii++){
        customerInWaitingChair[ii] = NO_CUSTOMER;
    }
}



BSState::~BSState()
{
    free(customerInWaitingChair);
}


/*
 *------------------------------------------------------------------
 *
 * set --
 *
 *         Set <this> according to the values passed in. 
 *         Makes copy of chair array.
 *
 *------------------------------------------------------------------
 */
void
BSState::set(int barberIsSleeping,
             int customerInBarberChair,
             int nchairs, 
             int *customerInWaitingChair)
{
    //
    // Don't forget to update set when new fields added
    //
    assert(sizeof(BSState) == 3*sizeof(int) + sizeof(int *));

    int ii;

    this->barberIsSleeping = barberIsSleeping;
    this->customerInBarberChair = customerInBarberChair;
    this->nWaitingChairs = nchairs;
    if(this->customerInWaitingChair != NULL){
        free(this->customerInWaitingChair);
    }
    this->customerInWaitingChair = (int *)malloc(nchairs * sizeof(int));
    for(ii = 0; ii < this->nWaitingChairs; ii++){
        this->customerInWaitingChair[ii] = customerInWaitingChair[ii];
    }
}


/*
 *------------------------------------------------------------------
 *
 * equals --
 *
 *         Return true if the argument equals this in all fields
 *
 *------------------------------------------------------------------
 */
int
BSState::equals(BSState *b)
{
    //
    // Don't forget to update equals when new fields added
    //
    assert(sizeof(BSState) == 3*sizeof(int) + sizeof(int *));

    if(b->barberIsSleeping != barberIsSleeping){
        return 0;
    }
    if(b->customerInBarberChair != customerInBarberChair){
        return 0;
    }
    if(b->nWaitingChairs != nWaitingChairs){
        return 0;
    }
    int ii;
    for(ii = 0; ii < nWaitingChairs; ii++){
        if(b->customerInWaitingChair[ii] != customerInWaitingChair[ii]){
            return 0;
        }
    }
    return 1;
}

/*
 *------------------------------------------------------------------
 *
 * print --
 *
 *         Print to stdout
 *
 *------------------------------------------------------------------
 */
void
BSState::print()
{
    //
    // Don't forget to update print when new fields added
    //
    assert(sizeof(BSState) == 3*sizeof(int) + sizeof(int *));

    int ii;
    printf("%s %d ", (barberIsSleeping ? "SLEEPING" : "AWAKE"),
           customerInBarberChair);
    for(ii = 0; ii < nWaitingChairs; ii++){
        printf("%d ", customerInWaitingChair[ii]);
    }
    printf("\n");
}


/*
 *------------------------------------------------------------------
 *
 * getCustCount --
 *
 *         Return the number of customers in the store.
 *
 *------------------------------------------------------------------
 */
int 
BSState::getCustCount()
{
    int ii;
    int count = 0;
    for(ii = 0; ii < nWaitingChairs; ii++){
        if(customerInWaitingChair[ii] != NO_CUSTOMER){
            count++;
        }
    }
    if(customerInBarberChair != NO_CUSTOMER){
        count++;
    }
    return count;
}



/*
 *------------------------------------------------------------------
 *
 * sanity --
 *
 *         Simple sanity checks on state
 *
 * DO NOT CHANGE THIS FUNCTION. If you want to add additional checks,
 * then add them to additionalChecks() 
 *
 *------------------------------------------------------------------
 */
void
BSState::sanity()
{

    //
    // DO NOT CHANGE THIS FUNCTION
    //
    this->custInBarberChairImpliesAwake();
    this->noDupCustomers();
    this->additionalChecks();
    //
    // DO NOT CHANGE THIS FUNCTION
    //
}


/*
 *------------------------------------------------------------------
 *
 * custInBarberChairImpliesAwake --
 *
 *         Check invariant
 *
 *------------------------------------------------------------------
 */
void 
BSState::custInBarberChairImpliesAwake()
{
    if(this->customerInBarberChair != NO_CUSTOMER){
        assert(this->barberIsSleeping == 0);
    }
}

/*
 *------------------------------------------------------------------
 *
 * noDupCustomers --
 *
 *         Check invariant
 *
 *------------------------------------------------------------------
 */
void 
BSState::noDupCustomers()
{
    int ii, jj;
    for(ii = 0; ii < this->nWaitingChairs; ii++){
        if(this->customerInWaitingChair[ii] != NO_CUSTOMER){
            assert(this->customerInWaitingChair[ii] != 
                   this->customerInBarberChair);
            for(jj = 0; jj < this->nWaitingChairs; jj++){
                if(ii != jj){
                    assert(this->customerInWaitingChair[ii] != 
                           this->customerInWaitingChair[jj]);
                }
            }
        }
    }
}


/*
 *------------------------------------------------------------------
 *
 * additionalChecks --
 *
 *         Add your own sanity checks here.
 *
 *------------------------------------------------------------------
 */
void
BSState::additionalChecks()
{

    // Add your own sanity checks here

}
