#ifndef _BSSTATE_H_
#define _BSSTATE_H_

const int NO_CUSTOMER = -1;
const int MAX_WAITING_CHAIRS = 20;

//
// Customer IDs smaller than 0 are reserved for special
// cases like "NO_CUSTOMER"
//
inline int isLegalCustomerId(int id){return (id < 0 ? 0 : 1);}


class BSState{
public:
    BSState(int nchairs);
    ~BSState();
    void sanity();
    void set(int barberIsSleeping,
             int customerInBarberChair,
             int nchairs, 
             int *customerInWaitingChair);
    int equals(BSState *b);
    void print();

    //
    // Notice that we never share BSState across threads
    // so no synchronization is needed
    //
    int getBarberIsSleeping(){return barberIsSleeping;}
    int getInBarberChair(){return customerInBarberChair;}
    int getCustCount();

    //
    // Feel free to add additional public or private methods.
    //
private:

    void additionalChecks();
    void custInBarberChairImpliesAwake();
    void noDupCustomers();

    //
    // Do not change any of the existing fields -- we need them 
    // for testing.
    //
    int barberIsSleeping;      // 1 = sleeping, 0 = awake
    int customerInBarberChair; // NO_CUSTOMER = no customer in chair
    int nWaitingChairs;        // number of waiting chairs in shop
    int *customerInWaitingChair; // NO_CUSTOMER = no 
                                 // customer in chair
    
    //
    // Feel free to add additional fields to aid your own testing.
    //

};


#endif
